﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NHibernate.Cfg;
using NHibernate;
using DataTransfer;
using NHibernate.Expression;


namespace DataAccessLayer
{
    public class NHibernateDataProvider
    {
        private ISession _session;
        private static ISession GetSession()
        {
            ISessionFactory sessionFactory = (new Configuration()).Configure().BuildSessionFactory();

            ISession session = sessionFactory.OpenSession();
            return session;
        }

        /// <summary>
        /// Initializes a new instance of the NHibernateDataProvider class.
        /// </summary>
        public NHibernateDataProvider()
        {
            _session = GetSession();
        }

        public IList<string> GetDistinctCustomerFirstnames()
        {
            return _session.CreateQuery("select distinct c.Firstname from Customer c").List<string>();
        }

        public IList<string> CRIT_GetDistinctCustomerFirstnames()
        {
            return _session.CreateCriteria(typeof(Customer))
                .SetProjection(Projections.Distinct(Projections.Property("Firstname")))
                .List<string>();
        }

        public IList<Customer> CRIT_GetCustomersOrderedByLastnames()
        {
            return _session.CreateCriteria(typeof(Customer))
                .AddOrder(new Order("Lastname", true))
                .List<Customer>();
        }


        public IList<CustomerFirstnameCounter> GetCustomerFirstnameCounts()
        {
            return _session.CreateQuery("select new CustomerFirstnameCounter(c.Firstname, count(c.Firstname)) from Customer c group by c.Firstname").List<CustomerFirstnameCounter>();
        }


        public IList<Customer> GetCustomersOrderedByLastnames()
        {
            return _session.CreateQuery("select from Customer c order by c.Lastname").List<Customer>();
        }


        public Customer GetCustomerById(int customerId)
        {
            return _session.Get<Customer>(customerId);
        }

        public IList<Customer> GetCustomersByFirstname(string firstname)
        {
            

            return _session.CreateQuery("select from Customer c where c.Firstname=:fn")
                .SetString("fn", firstname)
                .List<Customer>();
        }

        public IList<Customer> GetCustomersByFirstnameAndLastname(string firstname, string lastname)
        {
            

            return _session.CreateQuery("select from Customer c where c.Firstname=:fn and c.Lastname=:ln")
                .SetString("fn", firstname)
                .SetString("ln", lastname)
                .List<Customer>();

        }

        public IList<Customer> GetCustomersWithIdGreaterThan(int customerId)
        {
            

            return _session.CreateQuery("select from Customer c where c.CustomerId > :cid")
                .SetInt32("cid", customerId)
                .List<Customer>();
        }


        public IList<Customer> CRIT_GetCustomersByFirstname(string firstname)
        {


            return _session.CreateCriteria(typeof(Customer))
                .Add(new EqExpression("Firstname", firstname))
                .List<Customer>();
        }

        public IList<Customer> CRIT_GetCustomersByFirstnameAndLastname(string firstname, string lastname)
        {



            return _session.CreateCriteria(typeof(Customer))
                .Add(new EqExpression("Firstname", firstname))
                .Add(new EqExpression("Lastname", lastname))
                .List<Customer>();
        }

        public IList<Customer> CRIT_GetCutomersWithIdGreaterThan(int customerId)
        {

            return _session.CreateCriteria(typeof(Customer))
                .Add(new GtExpression("CustomerId", customerId))
                .List<Customer>();
        }


        public IList<Customer> GetCustomersByExample(Customer customerSample)
        {



            return _session.CreateCriteria(typeof(Customer))
                .Add(Example.Create(customerSample))
                .List<Customer>();

        }


    }
}
