﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using DataTransfer;

namespace DataAccessLayerTest
{
    [TestFixture]
    public class NHibernateDataProviderTest
    {
        private DataAccessLayer.NHibernateDataProvider _provider;

        [Test]
        public void CanGetCustomerById()
        {
            Assert.AreEqual(1, _provider.GetCustomerById(1).CustomerId);
        }

        [Test]
        public void CanGetCustomerByFirstname()
        {
            const string CUSTOMER_FIRSTNAME = "Steve";
            const int QTY_CUSTOMERS_EXPECTED = 3;

            IList<Customer> customers = _provider.GetCustomersByFirstname(CUSTOMER_FIRSTNAME);

            if (customers.Count == 0)
                Assert.Warning("YOU MAY NOT BE TESTING WHAT YOU THINK YOU ARE!");


            foreach (Customer c in customers)
            {
                Assert.AreEqual(CUSTOMER_FIRSTNAME, c.Firstname);
            }

            Assert.AreEqual(QTY_CUSTOMERS_EXPECTED, customers.Count);

        }

        [Test]
        public void CRIT_CanGetCustomerByFirstname()
        {
            const string CUSTOMER_FIRSTNAME = "Steve";
            const int QTY_CUSTOMERS_EXPECTED = 3;

            IList<Customer> customers = _provider.CRIT_GetCustomersByFirstname(CUSTOMER_FIRSTNAME);

            if (customers.Count == 0)
                Assert.Warning("YOU MAY NOT BE TESTING WHAT YOU THINK YOU ARE!");


            foreach (Customer c in customers)
            {
                Assert.AreEqual(CUSTOMER_FIRSTNAME, c.Firstname);
            }

            Assert.AreEqual(QTY_CUSTOMERS_EXPECTED, customers.Count);

        }

        [Test]
        public void CanGetCustomerByFirstnameAndLastname()
        {
            const string CUSTOMER_FIRSTNAME = "Steve";
            const string CUSTOMER_LASTNAME = "Bohlen";
            const int QTY_CUSTOMER_EXPECTED = 2;

            IList<Customer> customers = _provider.GetCustomersByFirstnameAndLastname(CUSTOMER_FIRSTNAME, CUSTOMER_LASTNAME);

            if (customers.Count == 0)
                Assert.Warning("YOU MAY NOT BE TESTING WHAT YOU THINK YOU ARE!");

            foreach (Customer c in customers)
            {
                Assert.AreEqual(CUSTOMER_FIRSTNAME, c.Firstname);
                Assert.AreEqual(CUSTOMER_LASTNAME, c.Lastname);
            }

            Assert.AreEqual(QTY_CUSTOMER_EXPECTED, customers.Count);
        }

        [Test]
        public void CRIT_CanGetCustomerByFirstnameAndLastname()
        {
            const string CUSTOMER_FIRSTNAME = "Steve";
            const string CUSTOMER_LASTNAME = "Bohlen";
            const int QTY_CUSTOMER_EXPECTED = 2;

            IList<Customer> customers = _provider.CRIT_GetCustomersByFirstnameAndLastname(CUSTOMER_FIRSTNAME, CUSTOMER_LASTNAME);

            if (customers.Count == 0)
                Assert.Warning("YOU MAY NOT BE TESTING WHAT YOU THINK YOU ARE!");

            foreach (Customer c in customers)
            {
                Assert.AreEqual(CUSTOMER_FIRSTNAME, c.Firstname);
                Assert.AreEqual(CUSTOMER_LASTNAME, c.Lastname);
            }

            Assert.AreEqual(QTY_CUSTOMER_EXPECTED, customers.Count);
        }

        [Test]
        public void CanGetCustomersWithIdGreaterThan()
        {
            IList<Customer> customers = _provider.GetCustomersWithIdGreaterThan(20);

            if (customers.Count == 0)
                Assert.Warning("YOU MAY NOT BE TESTING WHAT YOU THINK YOU ARE!");


            foreach (Customer c in customers)
            {
                Assert.GreaterThan(c.CustomerId, 20);
            }
        }

        [Test]
        public void CRIT_CanGetCustomersWithIdGreaterThan()
        {
            IList<Customer> customers = _provider.CRIT_GetCutomersWithIdGreaterThan(20);

            if (customers.Count == 0)
                Assert.Warning("YOU MAY NOT BE TESTING WHAT YOU THINK YOU ARE!");


            foreach (Customer c in customers)
            {
                Assert.GreaterThan(c.CustomerId, 20);
            }
        }


        [Test]
        public void CanGetCustomerByExample()
        {
            const string CUSTOMER_FIRSTNAME = "Steve";
            const int QTY_CUSTOMERS_EXPECTED = 2;
            const string CUSTOMER_LASTNAME = "Bohlen";
            Customer theCustomer = new Customer();


            theCustomer.Firstname = CUSTOMER_FIRSTNAME;

            theCustomer.Lastname = CUSTOMER_LASTNAME;

            IList<Customer> customers = _provider.GetCustomersByExample(theCustomer);

            if (customers.Count == 0)
                Assert.Warning("YOU MAY NOT BE TESTING WHAT YOU THINK YOU ARE!");


            foreach (Customer c in customers)
            {
                Assert.AreEqual(CUSTOMER_FIRSTNAME, c.Firstname);
                Assert.AreEqual(CUSTOMER_LASTNAME, c.Lastname);
            }

            Assert.AreEqual(QTY_CUSTOMERS_EXPECTED, customers.Count);
        }


        [Test]
        public void CanGetDistinctCustomerFirstnames()
        {
            IList<string> firstnames = _provider.GetDistinctCustomerFirstnames();

            if (firstnames.Count == 0)
                Assert.Warning("No Firstnames were found!");

            foreach (string fn in firstnames)
            {
                Assert.AreEqual(1, firstnames.Count<string>(x => x == fn));
            }

        }

        [Test]
        public void CanGetCustomersOrderedByLastname()
        {
            IList<Customer> customers = _provider.GetCustomersOrderedByLastnames();

            Customer priorCustomer = null;

            foreach (Customer c in customers)
            {
                if (priorCustomer != null)
                    Assert.GreaterEqualThan(string.Compare(c.Lastname, priorCustomer.Lastname), 0);

                priorCustomer = c;
            }
        }

        [Test]
        public void CanGetCountsOfCustomerFirsnames()
        {
            IList<CustomerFirstnameCounter> firstnameCounts = _provider.GetCustomerFirstnameCounts();

            Dictionary<string, int> expectedCounts = new Dictionary<string, int>();
            expectedCounts.Add("Steve", 3);
            expectedCounts.Add("Mike", 1);
            expectedCounts.Add("Peter", 1);
            expectedCounts.Add("Joe", 1);
            expectedCounts.Add("Yuliana", 1);

            foreach (CustomerFirstnameCounter item in firstnameCounts)
            {
                Assert.AreEqual(expectedCounts[item.Firstname], item.Count);
                Assert.IsTrue(expectedCounts.ContainsKey(item.Firstname));
            }
            
        }


        [Test]
        public void CRIT_CanGetCustomersOrderedByLastname()
        {
            IList<Customer> customers = _provider.CRIT_GetCustomersOrderedByLastnames();

            Customer priorCustomer = null;

            foreach (Customer c in customers)
            {
                if (priorCustomer != null)
                    Assert.GreaterEqualThan(string.Compare(c.Lastname, priorCustomer.Lastname), 0);

                priorCustomer = c;
            }
        }


        [Test]
        public void CRIT_CanGetDistinctCustomerFirstnames()
        {
            IList<string> firstnames = _provider.CRIT_GetDistinctCustomerFirstnames();

            if (firstnames.Count == 0)
                Assert.Warning("No Firstnames were found!");

            foreach (string fn in firstnames)
            {
                Assert.AreEqual(1, firstnames.Count<string>(x => x == fn));
            }

        }


        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            _provider = new DataAccessLayer.NHibernateDataProvider();
        }


    }
}
